import java.net.*;		// Socket
import java.io.*;		// IOException

public class GkClient implements Runnable {
	private GkGUI gui;
	private String host;
	private int port;
	
	// socket io to communicate with the gatekeeper
	private Socket socket;
	private InputStream istream;
	private OutputStream ostream;

	public GkClient() {
	}

	public void init(GkGUI _gui, String _host, int _port) {
		gui = _gui;
		host = _host;
		port = _port;
	}
	
	public void stop() {
		try {
			// cleanup
			if (istream != null)
				istream.close();
			istream = null;
			if (ostream != null)
				ostream.close();
			ostream = null;
			if (socket != null)
				socket.close();
			socket = null;
		}
		catch(IOException e) { return; };
	};

	public synchronized void run() {
		if (istream != null) {
			return;
		}
		try {
			gui.appendLog("Connecting to gatekeeper at " + host + ":" + port + "\n");
			socket = new Socket(host, port);
		}
		catch(Exception e) { gui.appendLog("Error: " + e + "\n"); return; };

		try {
			ostream = socket.getOutputStream();
			istream = socket.getInputStream();
			BufferedReader in = new BufferedReader(new InputStreamReader(istream));

			refresh();	// get initial gk status
			String inputLine;			
			while ((inputLine = in.readLine()) != null) {
				// append the event to log
				gui.appendLog(inputLine + "\n");
				
				// update endpoint buttons
				if (inputLine.startsWith("RCF")) {
					String epid = extractField(inputLine, 5);
					String alias = extractField(inputLine, 3);
					gui.addEndpoint(epid, alias);
				}
				else if (inputLine.startsWith("UCF")) {
					String epid = extractField(inputLine, 3);
					gui.removeEndpoint(epid);
				}
				else if (inputLine.startsWith("ACF")) {
					String epid = extractField(inputLine, 3);
					gui.setEndpointState(epid, Endpoint.TALKING);
				}
				else if (inputLine.startsWith("DCF")) {
					String epid = extractField(inputLine, 3);
					gui.setEndpointState(epid, Endpoint.AVAILABLE);
				}
			};

			stop();			
		}
		catch(IOException e) { };
		gui.appendLog("Disconnecting from gatekeeper\n");
	}
	
	public void refresh() {
		if (ostream != null) {
			try {
				ostream.write("PrintAllRegistrations\r\n".getBytes());
				ostream.write("PrintCurrentCalls\r\n".getBytes());
			}
			catch (Exception e) { gui.appendLog("Exception: " + e + "\n"); }
		}
	}
	
	public void disconnectEndpoint(String _epid) {
		if (ostream != null) {
			try {
				ostream.write(("DisconnectEndpoint " + _epid + "\r\n").getBytes());
			}
			catch (Exception e) { gui.appendLog("Exception: " + e + "\n"); }
		}
	}

	public void disconnectAlias(String _alias) {
		if (ostream != null) {
			try {
				ostream.write(("DisconnectAlias " + extractAliasName(_alias) + "\r\n").getBytes());
			}
			catch (Exception e) { gui.appendLog("Exception: " + e + "\n"); }
		}
	}
	
	public void unregisterAlias(String _alias) {
		if (ostream != null) {
			try {
				ostream.write(("UnregisterAlias " + extractAliasName(_alias) + "\r\n").getBytes());
			}
			catch (Exception e) { gui.appendLog("Exception: " + e + "\n"); }
		}
	}

	public void makeCall(String _epid, String _destination) {
		gui.appendLog("MakeCall: " + _epid + " -> " + _destination + "\n");
		if (ostream != null) {
			try {
				ostream.write(("MakeCall " + _epid + " " + _destination + "\r\n").getBytes());
			}
			catch (Exception e) { gui.appendLog("Exception: " + e + "\n"); }
		}
	}

	// extract a field (counting from one) from a pipe-seperated string ("field 1|field 2|field 3")
	private String extractField(String _msg, int _no) {
		String field = "";
		int start = 0;
		int end = 0;
		if (_msg.endsWith(";"))
			_msg = _msg.substring(0, _msg.length() - 1);
		_msg += "|";
		for (int i = 0; i < (_no - 1); i++) {
			start = _msg.indexOf("|", start) + 1;
		}
		for (int i = 0; i < _no; i++) {
			end = _msg.indexOf("|", end + 1);
		}
		try {
			field = _msg.substring(start, end);
			// gui.appendLog("Field " + _no + " = " + field + "\n");
		}
		finally {
			return field;
		}
	}

	// remove the type from the alias string
	private String extractAliasName(String _alias) {
		String aliasName = "";
		try {
			aliasName = _alias.substring(0, _alias.indexOf(":"));
		}
		finally {
			return aliasName;
		}
	}
}
